%% Code for Day 7 

% Add path to EEGLab and launch EEGLab before we begin (or use your
% shortcut on the shortcuts bar)

% Make sure you can access your sample data (sampleEEGdata.mat)
load '../sampleEEGdata.mat'; % change directory as needed


%% A few details from last class (power) before discussing today's topic (phase(
% 
%% Power law and Normalization via Baselining
% --------------------------------------------

    %% dB normalization (from Lecturelets code)

    activity = 1:.01:20; % activity
    baseline = 10; % baseline

    db = 10*log10(activity./ baseline );

    figure(1), clf
    plot(activity,activity,'linew',3)
    hold on
    plot(activity, db,'r','linew',3)
    legend({'"activity"','dB'})

    %% percent change
    pc = 100*( activity-baseline)./ baseline;

    figure(1), clf
    plot(activity,activity,'linew',3)
    hold on
    [hAx,hLine1,hLine2] = plotyy(activity, db,activity, pc);
    hLine1.Color = [1 0 1];
    hLine2.Color = [0 0 0];
    legend({'"activity"','dB', '%'})

   %% Choosing baselines 
   % --------------------------------------
    edit whichbaseline.m

    % Explore time windows for baseline; choose wisely and foolishly
    
    % Comment: Condition specific vs Condition-average




%% Temporal Downsampling following tf analysis
% --------------------------------------------
    edit postanalysis_ds.m
   
    % TF retains temporal resolution, but decreases temporal precision



%% Inter-trial Phase Clustering/Consistency (ITPC)
clear all; close all; clc

    %% Phase is best plotted in Polar Space

        % define two angles equidistant from 2*pi radians
        a = [0.2 2*pi-.2];

        figure
        % plot unit vectors defined by those angles
        polar([0 0; a],[0 0; 1 1])

    %% What is the mean phase angle -- ITPC (Inter-trial phase clustering)

        % Plot the mean vector incorrectly, then correctly
        hold on

        % plot a unit vector with the average angle -- INCORRECT
        polar([0 mean(a)],[0 1],'r')

        % plot the average vector -- CORRECT
        polar([0 angle(mean(exp(1i*a)))],[0 abs(mean(exp(1i*a)))],'m')

        % QUESTION: what is range of ITPC values?  

    %% illustration of ITPC with different clustering degrees

        % specify parameters
        circ_prop = 0.5; % proportion of the circle to fill (range 0-1)
        N = 100; % number of "trials"
        center_phase = pi;  % adjust to change anticipated mean angle

        % generate phase angle distribution
        % start with expected mean phase and add to it random numbers in
        % either direction (hence -0.5 to range from -0.5 to + 0.5)
        simdata = center_phase + ((rand(1,N)-0.5) * (2*pi) * circ_prop);

        % compute ITPC and preferred phase angle 
        % NOTE: no need to extract angle from complex signal since simdata is the angles
        % good old Euler!
        itpc      = abs(mean(exp(1i*simdata)));  
        prefAngle = angle(mean(exp(1i*simdata)));


        % and plot...
        figure(2), clf

        % as linear histogram
        subplot(3,3,4)
        hist(simdata,20)
        xlabel('Phase angle'), ylabel('Count')
        set(gca,'xlim',[0 2*pi])
        title([ 'Observed ITPC: ' num2str(itpc) ])

        % and as polar distribution
        subplot(1,2,2)
        polar([zeros(1,N); simdata],[zeros(1,N); ones(1,N)],'k')
        hold on
        h = polar([0 prefAngle],[0 itpc],'m');
        set(h,'linew',3)
        title([ 'Observed ITPC: ' num2str(itpc) ])

        %% ASSIGNMENT: ITPC is sample-size dependent part 1
            %
            
            % Edit code above:
                % Create a random phase distribution (hint: Fill the circle entirely)
                % rerun this code to see the change in ITPC from when the distribution was more tightly clustered
            
                % Edit sample size to make it larger or much larger
                % rerun the code to see change in ITPC 

            % QUESTION: under this assumption of random phase distribution, what is expected ITPC value?
            
            % QUESTION: under this assumption of random phase distribution, what is expected preferred angle?
            
        %% ASSIGNMENT: ITPC is sample-size dependent part 2
            % 
            
            % Calculate ITPC from a random phase distribution as a
               % function of sample size ranging from 2 to 10000 in any steps
               % you see fit.  
            
            % Get 10 estimates of ITPC at each sample size
            
            % Take the mean across the 10 estimates
            
            % Plot sample size (x Axis) by mean ITPC (Y Axis)
            
            % Compute spearman rank order correation (hint: doc corr) and
               % indicate the rank order correlation in the plot title
        
    %% Derive ITPC from real data and plot in TF space

        edit itpc.m  
        
        % QUESTION: how do we interpret this?
        % try another 
        
    %% impact of jitter on ITPC is greater than on Power
    
        edit jitter.m
        
    %% ASSIGNMENT
    %
    
    % Modify code in jitter.m to have no jitter, but instead have two
        % channels (and save as a new scipt)
    
    % Now compare the two channels side-by side for power and for ITPC
    
    % Do this for a few pairs of channels 
        % Are the patterns of results from ITPC and power generally similar or different? 
        % Do the results look more similar at some electrodes and less similar at other electrodes?


%% Total,Phase-Locked,and Non-Phase-Locked Power and ITPC
% ----------------------------------------------------------

        % Assumption: Non-phaselocked = Total - ERP
        edit nonphaselocked.m
        
        % ASSIGNMENT: Try a different channel
        
        % Example
        Trujillo_Allen_2007 = imread('Trujillo_Allen_2007.jpg');
        figure; 
        imshow(Trujillo_Allen_2007);
        
%% Surface Laplacian
% ----------------------------------------------------------
        
    %% Some basics
    edit laplacian.m
    
    %% ASSIGNMENT: Change the m parameter that will change the sharpness of the result using ERN sample data
    %
    close all; clear all; clc

    % Load Data
    load ERN_Data_LM_Ref
    
   
    % create topoplot of average 
    % At what time in msec? 
    time2plot = 40;
    % find time index
    timeidx = dsearchn(EEG.times',time2plot);
    % average data from this time point
    tempdata = squeeze(mean(EEG.data(:,timeidx,:),3));
    % plot voltage map (spatially unfiltered)
    figure; subplot(2,1,1);
    topoplot(tempdata,EEG.chanlocs,'plotrad',.53,'electrodes','off');
    title([ 'Voltage, ' num2str(time2plot) ' ms' ])
    colorbar

    % Perform Laplacian Transform and plot the result with a title in
    % second subplot
    % plot Laplacian map (spatially filtered)
    
    % a hint you might appreciate
    X = [EEG.chanlocs.X];
    Y = [EEG.chanlocs.Y];
    Z = [EEG.chanlocs.Z];

    % another hint
    % [surf_lap,G,H] = laplacian_perrinX(data,x,y,z[,leg_order,smoothing]);
    
    subplot(2,1,2)
    
    
    % Now alter the Legendre order parameter (try 1-10) and replot in the
    % second plot

    % Now alter the time to plot somewhere between 0 and 50 msec and plot
    
%% ASSIGNMENT: Integration of Laplacian, TF-Power and ITPC
% ----------------------------------------------------------

    % Using the ERN Sample Data, choose a site where the ERN is likely to be prominent
    
    % Using the ERN sample data at this site, compute the time-frequency power 
        % and ITPC on the raw data, and also on the Laplacian-transformed data
        
    % Plot a 2x2 matcix of tf plots with 
        % columns = voltage, Laplacian
        % rows = db-baselined power, ITPC
        
        
        