%% ASSIGNMENT: Integration of Laplacian, TF-Power and ITPC
% ----------------------------------------------------------
clear all; close all; clc

    % load data
    load ERN_Data_LM_Ref
    
    % Using the ERN Sample Data, choose a site where the ERN is likely to be prominent
    channel2use = 'Cz';

    % Choose baseline for ERN data (don't get too close to response)
    baseline_times = [-300 -100];
    baseline_idx = dsearchn(EEG.times',baseline_times');
    
    % Using the ERN sample data at this site, compute the time-frequency power 
        % and ITPC on the raw data, and also on the Laplacian-transformed data
        
    % Laplacian
    EEG_laplace = EEG;  % duplicate structure and the replace data (next)
    EEG_laplace.data = laplacian_perrinX(EEG.data,[EEG.chanlocs.X],[EEG.chanlocs.Y],[EEG.chanlocs.Z]);

    % setup wavelet parameters
    num_frex = 20;
    min_freq =  2;
    max_freq = 18;

    % set range for variable number of wavelet cycles
    range_cycles = [ 3 7 ];

    % other wavelet parameters
    frex = logspace(log10(min_freq),log10(max_freq),num_frex);
    wavecycles = logspace(log10(range_cycles(1)),log10(range_cycles(end)),num_frex);
    time = -2:1/EEG.srate:2;
    half_wave_size = (length(time)-1)/2;

    % FFT parameters
    nWave = length(time);
    nData = EEG.pnts*EEG.trials;
    nConv = nWave+nData-1;


    % FFT of data (doesn't change on frequency iteration)
    dataX = fft( reshape(EEG.data(strcmpi(channel2use,{EEG.chanlocs.labels}),:,:),1,nData) ,nConv);
    data_laplaceX = fft( reshape(EEG_laplace.data(strcmpi(channel2use,{EEG_laplace.chanlocs.labels}),:,:),1,nData) ,nConv);

    % initialize output time-frequency data
    tf = zeros(2,num_frex,EEG.pnts);  % 2 is for power(1), phase(2)
    tf_laplace = zeros(2,num_frex,EEG_laplace.pnts);

    % loop over frequencies
    for fi=1:num_frex
        
        % create wavelet and get its FFT
        s = wavecycles(fi)/(2*pi*frex(fi));
        wavelet  = exp(2*1i*pi*frex(fi).*time) .* exp(-time.^2./(2*s^2));
        waveletX = fft(wavelet,nConv);
        
        % run convolution
        as = ifft(waveletX.*dataX,nConv);
        as = as(half_wave_size+1:end-half_wave_size);
        as = reshape(as,EEG.pnts,EEG.trials);

        as_laplace = ifft(waveletX.*data_laplaceX,nConv);
        as_laplace = as_laplace(half_wave_size+1:end-half_wave_size);
        as_laplace = reshape(as_laplace,EEG_laplace.pnts,EEG_laplace.trials);

        
        % compute Power and ITPC  
        tf(1,fi,:) =   mean(abs(as).^2 ,2);
        tf(2,fi,:) =   abs(mean(exp(1i*angle(as)),2));
        tf_laplace(1,fi,:) =   mean(abs(as_laplace).^2 ,2);
        tf_laplace(2,fi,:) =   abs(mean(exp(1i*angle(as_laplace)),2));

       
    end

       % dB baseline the power
        tf_activity = tf(1,:,:);
        tf_baseline = mean(tf(1,:,baseline_idx(1):baseline_idx(2)) ,3);
        tf_laplace_activity = tf_laplace(1,:,:);
        tf_laplace_baseline = mean(tf_laplace(1,:,baseline_idx(1):baseline_idx(2)) ,3);
        % decibel
        tf(1,:,:) = 10*log10( bsxfun(@rdivide, tf_activity, tf_baseline) );
        tf_laplace(1,:,:) = 10*log10( bsxfun(@rdivide, tf_laplace_activity, tf_laplace_baseline) );
   
    % Plot a 2x2 matcix of tf plots with 
        % columns = voltage, Laplacian
        % rows = db-baselined power, ITPC
    dblim = [-5 5];
    itpclim = [0 .7];

    figure;
    subplot(221); % Voltage, power
    contourf(EEG.times,frex,squeeze(tf(1,:,:)),40,'linecolor','none')
    set(gca,'clim',dblim,'ydir','normal','xlim',[-300 800])
    title('Voltage TF Power -- dB baseline')
    subplot(222); % CSD, power
    contourf(EEG.times,frex,squeeze(tf_laplace(1,:,:)),40,'linecolor','none')
    set(gca,'clim',dblim,'ydir','normal','xlim',[-300 800])
    title('CSD TF Power -- dB baseline')
    subplot(223); % Voltage, ITPC)
    contourf(EEG.times,frex,squeeze(tf(2,:,:)),40,'linecolor','none')
    set(gca,'clim',itpclim,'ydir','normal','xlim',[-300 800])
    title('Voltage ITPC')
    subplot(224); % Voltage, ITPC)
    contourf(EEG.times,frex,squeeze(tf_laplace(2,:,:)),40,'linecolor','none')
    set(gca,'clim',itpclim,'ydir','normal','xlim',[-300 800])
    title('CSD ITPC')
    
    
    