% largely the same code as in mikexcohen@gmail.com
clear all; close all; clc

%% Compute and plot TF-ITPC for one electrode

load '../sampleEEGdata.mat'

% wavelet parameters
num_frex = 40;
min_freq =  2;
max_freq = 30;

channel2use = 'Oz';

% set range for variable number of wavelet cycles
range_cycles = [ 4 10 ];

% other wavelet parameters
frex = logspace(log10(min_freq),log10(max_freq),num_frex);
wavecycles = logspace(log10(range_cycles(1)),log10(range_cycles(end)),num_frex);
time = -2:1/EEG.srate:2;
half_wave_size = (length(time)-1)/2;

% FFT parameters
nWave = length(time);
nData = EEG.pnts*EEG.trials;
nConv = nWave+nData-1;


% FFT of data (doesn't change on frequency iteration)
dataX = fft( reshape(EEG.data(strcmpi(channel2use,{EEG.chanlocs.labels}),:,:),1,nData) ,nConv);

% initialize output time-frequency data
tf = zeros(num_frex,EEG.pnts);

% loop over frequencies
for fi=1:num_frex
    
    % create wavelet and get its FFT
    s = wavecycles(fi)/(2*pi*frex(fi));
    wavelet  = exp(2*1i*pi*frex(fi).*time) .* exp(-time.^2./(2*s^2));
    waveletX = fft(wavelet,nConv);
    
    % run convolution
    as = ifft(waveletX.*dataX,nConv);
    as = as(half_wave_size+1:end-half_wave_size);
    as = reshape(as,EEG.pnts,EEG.trials);
    
    % compute ITPC  (it's the absolute length of the mean Eulerized phase 
    % angle across trials: abs, mean, euler, angle)
      % For each frequency, this will be a time-series of ITPC values 
    tf(fi,:) =   abs(mean(exp(1i*angle(as)),2));
    
    % The wrong way to do it -- uncomment line to see what happens
    % tf(fi,:) =   mean(abs(exp(1i*angle(as))),2);
    
end

% plot results
figure(1), clf
contourf(EEG.times,frex,tf,40,'linecolor','none')
set(gca,'clim',[0 .6],'ydir','normal','xlim',[-300 1000])
title(['ITPC at site ' channel2use])
colorbar


%% end.
