%% Figure 19.9 -- code from Mike X Cohen
clear all; close all; clc

load('../sampleEEGdata.mat'); % change directory as needed

% pick electrode and frequencies
chan2plot   = 'Oz';
frequencies = 1:40;

% how many samples to jitter?
n_samples_to_jitter = 10; % 10 samples at 256 Hz (up to 39 msec)

% definte convolution parameters
time          = -EEG.pnts/EEG.srate/2:1/EEG.srate:EEG.pnts/EEG.srate/2-1/EEG.srate;
n_wavelet     = EEG.pnts;
n_data        = EEG.pnts*EEG.trials;
n_convolution = n_wavelet+n_data-1;
n_conv_pow2   = pow2(nextpow2(n_convolution));

plotlegends={'without jitter';'with jitter'};

figure

baselinetime = [ -300 -100 ];
baseidx=dsearchn(EEG.times',baselinetime(1)):dsearchn(EEG.times',baselinetime(2));

for simuli=1:2
    
    % add time jitter (or not)
    tempdat = squeeze(EEG.data(strcmpi(chan2plot,{EEG.chanlocs.labels}),:,:)); % time by trials for one channel
    for ti=1:size(tempdat,2)
        timejitter = ceil(rand*n_samples_to_jitter)*(simuli-1); % (when simuli==1, timejitter==0)
                                               % when jittering, between 0
                                               % n samples
        tempdat(:,ti) = tempdat([ timejitter+1:end 1:timejitter ],ti);  % shifts data forward timejitter points and puts the prior points at the end
    end
    
    % get FFT of data
    eegfft = fft(reshape(tempdat,1,[]),n_conv_pow2);
    
    % initialize
    itpc = zeros(length(frequencies),EEG.pnts);
    powr = zeros(length(frequencies),EEG.pnts);
    
    for fi=1:length(frequencies)
        
        centerfreq = frequencies(fi);
        
        wavelet = exp(2*1i*pi*centerfreq.*time) .* exp(-time.^2./(2*((4/(2*pi*centerfreq))^2)))/centerfreq;
        
        % convolution
        eegconv = ifft(fft(wavelet,n_conv_pow2).*eegfft);
        eegconv = eegconv(1:n_convolution);
        eegconv = reshape(eegconv(floor((EEG.pnts-1)/2):end-1-ceil((EEG.pnts-1)/2)),EEG.pnts,EEG.trials);
        
        % compute and store itpc and power
        itpc(fi,:) = abs(mean(exp(1i*angle(eegconv)),2));
        powr(fi,:) = mean(abs(eegconv).^2,2);
        powr(fi,:) = 10*log10(powr(fi,:)./mean(powr(fi,baseidx),2));
    end
    
    subplot(2,2,simuli)
    contourf(EEG.times,frequencies,itpc,40,'linecolor','none')
    set(gca,'clim',[.1 .5],'xlim',[-200 1000])
    xlabel('Time (ms)')
    ylabel('Frequency (Hz)')
    title([ 'ITPC ' plotlegends{simuli} ])
    
    subplot(2,2,simuli+2)
    contourf(EEG.times,frequencies,powr,40,'linecolor','none')
    set(gca,'clim',[-3 3],'xlim',[-200 1000])
    xlabel('Time (ms)')
    ylabel('Frequency (Hz)')
    title([ 'DB-power ' plotlegends{simuli} ])
end
